# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30 import interface


class TestInterface(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.target = interface.Interface(self.client)
        self.url_prefix = "/axapi/v3/interface/"

    def test_interface_get_list(self):
        self.target.get_list()
        self.client.http.request.assert_called_with("GET", self.url_prefix, {}, mock.ANY, axapi_args=None,
                                                    max_retries=None, timeout=mock.ANY)

    def test_interface_get(self):
        self.target.get()
        self.client.http.request.assert_called_with("GET", self.url_prefix, {}, mock.ANY, axapi_args=None,
                                                    max_retries=None, timeout=mock.ANY)

    def _test_interface_create_dhcp(self, dhcp=True):
        expected = 1 if dhcp else 0
        ifnum = 1
        expected_payload = {self.target.iftype: {'ip': {'dhcp': expected}, 'ifnum': ifnum}}
        self.target.create(ifnum, dhcp=dhcp)
        self.client.http.request.assert_called_with("POST", self.url_prefix,  # URL + ifnum expected
                                                    expected_payload, mock.ANY, axapi_args=None,
                                                    max_retries=None, timeout=mock.ANY)

    def test_interface_create_dhcp_negative(self):
        self._test_interface_create_dhcp(False)

    def test_interface_create_dhcp_positive(self):
        self._test_interface_create_dhcp()

    def test_interface_create_ipaddress(self):
        ifnum = 1
        ip_address = "128.0.0.1"
        ip_netmask = "255.255.255.0"
        self.target.create(ifnum, dhcp=False, ip_address=ip_address, ip_netmask=ip_netmask)
        self.client.http.request.assert_called_with("POST", self.url_prefix, mock.ANY, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_interface_delete(self):
        ifnum = 1
        self.target.delete(1)
        self.client.http.request.assert_called_with("DELETE", self.url_prefix + str(ifnum),
                                                    mock.ANY, mock.ANY, axapi_args=None, max_retries=None,
                                                    timeout=mock.ANY)

    def test_interface_update(self):
        ifnum = 1
        ip_address = "128.0.0.1"
        ip_netmask = "255.255.255.0"

        self.target.update(ifnum, dhcp=False, ip_address=ip_address, ip_netmask=ip_netmask)

        self.client.http.request.assert_called_with("POST", self.url_prefix + str(ifnum),
                                                    mock.ANY, mock.ANY, axapi_args=None, max_retries=None,
                                                    timeout=mock.ANY)

    def test_interface_enable_positive(self):
        ifnum = 1
        ip_address = "128.0.0.1"
        ip_netmask = "255.255.255.0"

        self.target.update(ifnum, dhcp=False, ip_address=ip_address, ip_netmask=ip_netmask,
                           enable=True)

        ((method, url, params, header), kwargs) = self.client.http.request.call_args
        self.assertEqual("enable", params[self.target.iftype]["action"])

        self.client.http.request.assert_called_with("POST", self.url_prefix + str(ifnum),
                                                    mock.ANY, mock.ANY, axapi_args=None, max_retries=None,
                                                    timeout=mock.ANY)

    def test_interface_enable_negative(self):
        ifnum = 1
        ip_address = "128.0.0.1"
        ip_netmask = "255.255.255.0"

        self.target.update(ifnum, dhcp=False, ip_address=ip_address, ip_netmask=ip_netmask,
                           enable=False)

        ((method, url, params, header), kwargs) = self.client.http.request.call_args

        self.assertEqual("disable", params[self.target.iftype]["action"])

        self.client.http.request.assert_called_with("POST", self.url_prefix + str(ifnum),
                                                    mock.ANY, mock.ANY, axapi_args=None, max_retries=None,
                                                    timeout=mock.ANY)


class TestEthernetInterface(TestInterface):
        def setUp(self):
            super(TestEthernetInterface, self).setUp()
            self.target = interface.EthernetInterface(self.client)
            self.url_prefix = "{0}{1}/".format(self.url_prefix, self.target.iftype)


class TestManagementInterface(TestInterface):
        def setUp(self):
            super(TestManagementInterface, self).setUp()
            self.target = interface.ManagementInterface(self.client)
            self.url_prefix = "{0}{1}/".format(self.url_prefix, self.target.iftype)



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client.v30 import vlan


class TestVlan(unittest.TestCase):
    def setUp(self):
        self.client = mock.MagicMock()
        self.target = vlan.Vlan(self.client)
        self.url_prefix = "/axapi/v3/network/vlan"
        self.vlan_id = 1
        self.expected_payload = {"vlan": {"vlan-num": self.vlan_id}}

    def test_interface_get_list(self):
        self.target.get_list()
        self.client.http.request.assert_called_with("GET", self.url_prefix, {}, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_interface_get(self):
        self.target.get(self.vlan_id)
        self.client.http.request.assert_called_with(
            "GET", '{0}/{1}'.format(self.url_prefix, self.vlan_id), {}, mock.ANY,
            axapi_args=None, max_retries=None, timeout=mock.ANY
        )

    def test_vlan_create_shared(self):
        self.target.create(self.vlan_id, shared_vlan=True, untagged_eths=[], untagged_trunks=[],
                           tagged_eths=[], tagged_trunks=[], veth=False, lif=None)

        ep = self.expected_payload
        ep['vlan']['shared-vlan'] = True
        self.client.http.request.assert_called_with("POST", self.url_prefix, ep, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_vlan_create_untagged_eths(self):
        untagged_eths = [{'untagged-ethernet-start': 2, 'untagged-ethernet-end': 2}]
        vlan._build_range_list = mock.Mock(return_value={'untagged-eth-list': untagged_eths})
        self.target.create(self.vlan_id, shared_vlan=False, untagged_eths=[2], untagged_trunks=[],
                           tagged_eths=[], tagged_trunks=[], veth=False, lif=None)

        ep = self.expected_payload
        ep['vlan']['untagged-eth-list'] = untagged_eths
        self.client.http.request.assert_called_with("POST", self.url_prefix, ep, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_vlan_create_untagged_trunks(self):
        untagged_trunks = [{'untagged-trunk-start': 2, 'untagged-trunk-end': 2}]
        vlan._build_range_list = mock.Mock(return_value={'untagged-trunk-list': untagged_trunks})
        self.target.create(self.vlan_id, shared_vlan=False, untagged_eths=[],
                           untagged_trunks=[2], tagged_eths=[],
                           tagged_trunks=[], veth=None, lif=None)

        ep = self.expected_payload
        ep['vlan']['untagged-trunk-list'] = untagged_trunks
        self.client.http.request.assert_called_with("POST", self.url_prefix, ep, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_vlan_create_tagged_eths(self):
        tagged_eths = [{'tagged-ethernet-start': 2, 'tagged-ethernet-end': 2}]
        vlan._build_range_list = mock.Mock(return_value={'tagged-eth-list': tagged_eths})
        self.target.create(self.vlan_id, shared_vlan=False, untagged_eths=[], untagged_trunks=[],
                           tagged_eths=[2], tagged_trunks=[], veth=False, lif=None)

        ep = self.expected_payload
        ep['vlan']['tagged-eth-list'] = tagged_eths
        self.client.http.request.assert_called_with("POST", self.url_prefix, ep, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_vlan_create_tagged_trunks(self):
        tagged_trunks = [{'tagged-trunk-start': 2, 'tagged-trunk-end': 2}]
        vlan._build_range_list = mock.Mock(return_value=tagged_trunks)
        self.target.create(self.vlan_id, shared_vlan=False, untagged_eths=[], untagged_trunks=[],
                           tagged_eths=[], tagged_trunks=[2], veth=False, lif=None)

        ep = self.expected_payload
        ep['vlan']['tagged-trunk-list'] = tagged_trunks
        self.client.http.request.assert_called_with("POST", self.url_prefix, ep, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_vlan_create_ve(self):
        self.target.create(self.vlan_id, shared_vlan=False, untagged_eths=[], untagged_trunks=[],
                           tagged_eths=[], tagged_trunks=[], veth=True, lif=None)

        ep = self.expected_payload
        ep['vlan']['ve'] = 1
        self.client.http.request.assert_called_with("POST", self.url_prefix, ep, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_vlan_create_lif(self):
        self.target.create(self.vlan_id, shared_vlan=False, untagged_eths=[], untagged_trunks=[],
                           tagged_eths=[], tagged_trunks=[], veth=False, lif=6)

        ep = self.expected_payload
        ep['vlan']['untagged-lif'] = 6
        self.client.http.request.assert_called_with("POST", self.url_prefix, ep, mock.ANY,
                                                    axapi_args=None, max_retries=None, timeout=mock.ANY)

    def test_vlan_delete(self):
        self.target.delete(self.vlan_id)
        self.client.http.request.assert_called_with(
            "DELETE", '{0}/{1}'.format(self.url_prefix, self.vlan_id), mock.ANY, mock.ANY,
            axapi_args=None, max_retries=None, timeout=mock.ANY
        )



#!/usr/bin/env python2

# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import print_function

import acos_client

import argparse
import os.path
import random
import sys
import traceback
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

sys.path.append(".")


partitions = ['shared', 'p1', 'p2', 'ipv6', 'ipv4-ipv6', 'ipv6-ipv4']

parser = argparse.ArgumentParser(description='acos-client smoke test')
parser.add_argument('host')
parser.add_argument('--port', type=int, default=443)
parser.add_argument('--protocol', default='https')
parser.add_argument('--user', default='admin')
parser.add_argument('--password', default='a10')
parser.add_argument('--axapi-version', required=True, choices=['2.1', '3.0'])
parser.add_argument('--partition', default='shared', choices=partitions + ['all'])
ARGS = parser.parse_args()


partition_map = {
    'shared': {
        'name': 'shared',
        's1': '192.168.2.254',
        'vip1': '192.168.2.250',
        'vip2': '192.168.2.249',
        'vip3': '192.168.2.248',
        'vip4': '192.168.2.247'
    },
    'p1': {
        'name': 'p1',
        's1': '192.168.2.244',
        'vip1': '192.168.2.240',
        'vip2': '192.168.2.239',
        'vip3': '192.168.2.238',
        'vip4': '192.168.2.237'
    },
    'p2': {
        'name': 'p2',
        's1': '192.168.2.234',
        'vip1': '192.168.2.230',
        'vip2': '192.168.2.229',
        'vip3': '192.168.2.228',
        'vip4': '192.168.2.227'
    },
    'ipv6': {
        'name': 'ipv6',
        's1': '2001:db8::1001',
        'vip1': '2001:db8:feed::5001',
        'vip2': '2001:db8:feed::5002',
        'vip3': '2001:db8:feed::5003',
        'vip4': '2001:db8:feed::5004',
    },
    'ipv4-ipv6': {
        'name': 'ipv4-ipv6',
        's1': '192.168.46.254',
        'vip1': '2001:db8:ee::4601',
        'vip2': '2001:db8:ee::4602',
        'vip3': '2001:db8:ee::4603',
        'vip4': '2001:db8:ee::4604',
    },
    'ipv6-ipv4': {
        'name': 'ipv6-ipv4',
        's1': '2001:db8:ee::6401',
        'vip1': '192.168.64.1',
        'vip2': '192.168.64.2',
        'vip3': '192.168.64.3',
        'vip4': '192.168.64.4'
    },
}


class Nope(Exception):
    pass


def get_client(h, password=None):
    p = password or h['password']
    c = acos_client.Client(h['host'], h['axapi'], h['user'], p,
                           port=h['port'],
                           protocol=h['protocol'])
    return c


def get_cert_from_path(cpath):
    if not os.path.exists(cpath):
        raise Nope("Can't open {0}".format(cpath))

    with open(cpath) as f:
        cert_data = f.read()

    filename = os.path.split(cpath)[1]

    rv = {
        "file": filename,
        "cert": cert_data,
        "size": len(cert_data),
    }

    return rv


def create_key(key_path, c):
    key = {
        "cert": None,
        "file": None,
        "size": 0,
        "action": "import"
    }

    try:
        c.file.ssl_key.delete(key_path)
    except acos_client.errors.NotFound:
        print("key not found, that's OK")

    key_data = get_cert_from_path(key_path)
    key.update(key_data)

    try:
        c.file.ssl_key.create(key_path,
                              key_data["cert"],
                              len(key_data["cert"]),
                              action="import")
    except acos_client.errors.Exists:
        print("Got exists... should have been deleted in previous.")

    try:
        key_get = c.file.ssl_key.get(key_path)
    except acos_client.errors.NotFound:
        raise Nope("Key retrieval failed.")

    return key_get


def create_cert(cert_path, c):
    cert = {
        "cert": None,
        "file": None,
        "size": 0,
        "certificate_type": "pem",
        "action": "import"
    }

    try:
        c.file.ssl_cert.delete(cert_path)
    except acos_client.errors.NotFound:
        print("got not found, that's OK")

    cert_data = get_cert_from_path(cert_path)
    cert.update(cert_data)

    try:
        c.file.ssl_cert.create(cert_path,
                               cert_data["cert"],
                               len(cert_data["cert"]),
                               certificate_type="pem",
                               action="import")
    except acos_client.errors.Exists:
        print("Got exists... should have been deleted in previous.")
    try:
        cert_get = c.file.ssl_cert.get(cert_path)
    except acos_client.errors.NotFound:
        raise Nope("Certificate retrieval failed.")

    return cert_get


def run_all(ax, partition, pmap):
    print("=============================================================")
    print("=============================================================")
    print("=============================================================")
    print("=============================================================")
    print("RUNNING AGAINST ACOS HOST ", ax)

    print("=============================================================")
    print("")
    print("Basic login/get info twice")
    c = get_client(ax)

    print("WAIT FOR CONNECT")
    c.wait_for_connect()
    print("WAIT FOR CONNECT END")

    r = c.system.information()
    print("LIBRARY RESPONSE = %s" % r)

    print("=============================================================")
    print("")
    print("About to do an authenticate with a bad password")
    try:
        c = get_client(ax, password='badpass')
        c.system.information()
    except acos_client.errors.InvalidSessionID:
        print("got Invalid Session ID, good")
    except acos_client.errors.AuthenticationFailure:
        print("got bad auth exception, good")
    else:
        sys.stdout.flush()
        raise Nope()

    print("=============================================================")
    print("")
    print("About to do a good close")
    c = get_client(ax)
    c.system.information()
    c.session.close()

    print("=============================================================")
    print("")
    print("About to do a close with no session id")
    c = get_client(ax)
    c.session.close()
    if c.session.session_id is not None:
        print("ERROR: session id is not none")
        print("ERROR: ", c.session.session_id)
        raise Nope()
    print("here comes the close with no id...")
    c.session.close()

    print("=============================================================")
    print("")
    print("About to do a close with bad session id")
    try:
        c.session.session_id = 'bad_session_id'
        c.session.close()
    except acos_client.errors.InvalidSessionID:
        print("got Invalid Session ID, good")
    else:
        if ARGS.axapi_version == "3.0":
            raise Nope()

    # print("=============================================================")
    # print("")
    # print("About to do a get info with bad session id")
    # c.session.session_id = 'bad_session_id'
    # try:
    #     c.system.information()
    # except acos_client.errors.InvalidSessionID:
    #     print("got invalid session error, good")
    # else:
    #     raise Nope()

    # Get a fresh client

    c = get_client(ax)
    c.system.information()

    # c.ha.sync('172.18.61.27', 'admin', 'a10')

    print("=============================================================")
    print("")
    print(" PARTITIONS!!! ")

    print("=============================================================")
    print("")
    print("About to search for partition ", partition)

    p_exists = c.system.partition.exists(partition)
    print("LIBRARY RESPONSE = %s" % p_exists)

    print("=============================================================")
    print("")
    print("About to make partition %s active (not exist, if not shared)" % partition)

    try:
        c.system.partition.active(partition)
    except acos_client.errors.NotFound:
        pass

    print("=============================================================")
    print("")
    print("About to create partition", partition)

    if not p_exists:
        c.system.partition.create(partition)

    try:
        c.system.partition.create(partition)
    except acos_client.errors.Exists:
        pass

    print("=============================================================")
    print("")
    print("About to make partition %s active" % partition)

    c.system.partition.active(partition)

    print("=============================================================")
    print("")
    print("Write mem")
    c = get_client(ax)
    print("A")
    c.system.action.write_memory()

    c_tmpl = "ssl_client"
    s_tmpl = "ssl_server"
    c_cert = "client_cert.pem"
    s_cert = "server_cert.pem"
    c_key = "client_key.pem"
    s_key = "server_key.pem"

    print("=============================================================")
    print("")
    print("SSL Certificate Create")

    if ARGS.axapi_version == "3.0":
        print("Create Client Certificate")
        create_cert(c_cert, c)
        print("Create Server Certificate")
        create_cert(s_cert, c)
        print("Create Client Key")
        create_key(c_key, c)
        print("Create Server Key")
        create_key(s_key, c)

        print("=============================================================")

        print("=============================================================")
        print("")
        print("Client SSL Create")

        try:
            c.slb.template.client_ssl.delete(c_tmpl)
        except acos_client.errors.NotFound:
            print("got not found, OK")

        c.slb.template.client_ssl.create(c_tmpl, c_cert, c_key)
        c.slb.template.client_ssl.get(c_tmpl)
        try:
            c.slb.template.client_ssl.create(c_tmpl, c_cert, c_key)
        except acos_client.errors.Exists:
            print("got already exists error, good")
        c.slb.template.client_ssl.update(c_tmpl, c_cert, c_key)

        try:
            c.slb.template.client_ssl.update("sns1", "cert1", "cert1")

        except acos_client.errors.NotFound:
            print("got not found, good")
        c.slb.template.client_ssl.delete(c_tmpl)
        c.slb.template.client_ssl.delete(c_tmpl)
        try:
            c.slb.template.client_ssl.get(c_tmpl)
        except acos_client.errors.NotFound:
            print("got not found, good")

        print("=============================================================")
        print("")
        print("Server SSL Create")
        try:
            c.slb.template.server_ssl.delete(s_tmpl)
        except acos_client.errors.NotFound:
            print("got not found, OK")

        c.slb.template.server_ssl.create(s_tmpl, s_cert, s_key)
        c.slb.template.server_ssl.get(s_tmpl)
        try:
            c.slb.template.server_ssl.create(s_tmpl, s_cert, s_key)
        except acos_client.errors.Exists:
            print("got already exists error, good")
        c.slb.template.server_ssl.update(s_tmpl, s_cert, s_key)
        try:
            c.slb.template.server_ssl.update("sns1", "cert1", "cert1")
        except acos_client.errors.NotFound:
            print("got not found, good")

        c.slb.template.server_ssl.delete(s_tmpl)
        try:
            c.slb.template.server_ssl.get(s_tmpl)
        except acos_client.errors.NotFound:
            print("got not found, good")

    else:
        # This test is probably going to fail.  2.1 cert uploading support isn't there
        print("Certificate upload not supported in 2.1")
        print("Skipping Client SSL Create Test")
        print("Skipping Server SSL Create Test")

    print("=============================================================")
    print("")
    print("Server Create")
    c.slb.server.delete("foobar")
    c.slb.server.create("foobar", pmap['s1'])
    r = c.slb.server.get("foobar")
    print("LIBRARY RESPONSE = %s" % r)
    try:
        c.slb.server.create("foobar", pmap['s1'])
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.server.delete("foobar")
    c.slb.server.delete("foobar")
    try:
        c.slb.server.get("foobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.server.create("foobar", pmap['s1'])

    print("=============================================================")
    print("")
    print("SG Create")
    # temp -- odd that we have to delete this vport
    try:
        c.slb.virtual_server.vport.delete(
            "vip3", "vip3_VPORT", c.slb.virtual_server.vport.HTTP, 80)
    except acos_client.errors.NotFound:
        print("vip3 doesn't exist, that's OK")

    # temp -- odd that we have to delete this vport
    try:
        c.slb.service_group.delete("pfoobar")
    except acos_client.errors.NotFound:
        print("sg pfoobar doesn't exist, that's OK")

    c.slb.service_group.create("pfoobar", c.slb.service_group.TCP,
                               c.slb.service_group.ROUND_ROBIN)
    r = c.slb.service_group.get("pfoobar")
    print("LIBRARY RESPONSE = %s" % r)
    try:
        c.slb.service_group.create("pfoobar", c.slb.service_group.TCP,
                                   c.slb.service_group.ROUND_ROBIN)
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.service_group.update("pfoobar", c.slb.service_group.TCP,
                               c.slb.service_group.LEAST_CONNECTION)
    try:
        c.slb.service_group.update("pnfoobar", c.slb.service_group.TCP,
                                   c.slb.service_group.LEAST_CONNECTION)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.service_group.delete("pfoobar")
    c.slb.service_group.delete("pfoobar")
    try:
        c.slb.service_group.get("pfoobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.service_group.create("pfoobar", c.slb.service_group.TCP,
                               c.slb.service_group.ROUND_ROBIN)

    print("=============================================================")
    print("")
    print("VIP Create")
    c.slb.virtual_server.delete("vip3")
    c.slb.virtual_server.create("vip3", pmap['vip3'])
    c.slb.virtual_server.get("vip3")

    c.slb.virtual_server.delete("vfoobar")
    c.slb.virtual_server.create("vfoobar", pmap['vip1'])
    r = c.slb.virtual_server.get("vfoobar")
    print("LIBRARY RESPONSE = %s" % r)
    r = c.slb.virtual_server.all()
    print("LIBRARY RESPONSE = %s" % r)
    try:
        c.slb.virtual_server.create("vfoobar", pmap['vip1'])
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    try:
        c.slb.virtual_server.stats("vfoobar")
    except Exception:
        pass
    try:
        oper = c.slb.virtual_server.oper("vfoobar")
        print("OPERATIONAL STATS (VIRTUAL SERVER):")
        print(oper)
    except Exception as ex:
        print(ex)
    c.slb.virtual_server.delete("vfoobar")
    c.slb.virtual_server.delete("vfoobar")
    try:
        c.slb.virtual_server.get("vfoobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()

    c.slb.virtual_server.vport.delete("vip3", "vip3_VPORT",
                                      c.slb.virtual_server.vport.HTTP, 80)
    c.slb.virtual_server.vport.create("vip3", "vip3_VPORT",
                                      service_group_name="pfoobar",
                                      protocol=c.slb.virtual_server.vport.HTTP,
                                      port='80')
    try:
        c.slb.virtual_server.vport.get("vip3",
                                       "vip3_VPORT",
                                       protocol=c.slb.virtual_server.vport.HTTP,
                                       port=80)
    except Exception:
        pass
    try:
        c.slb.virtual_server.vport.create(
            "vip3", "vip3_VPORT",
            service_group_name="pfoobar",
            protocol=c.slb.virtual_server.vport.HTTP,
            port='80')
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.virtual_server.vport.delete("vip3", "vip3_VPORT",
                                      c.slb.virtual_server.vport.HTTP, 80)
    c.slb.virtual_server.vport.delete("vip3", "vip3_VPORT",
                                      c.slb.virtual_server.vport.HTTP, 80)

    print("=============================================================")
    print("")
    print("HM Create")
    c.slb.hm.delete("hnfoobar")
    c.slb.hm.delete("hfoobar")
    c.slb.hm.create("hfoobar", c.slb.hm.HTTP, 5, 5, 5, 'GET', '/', '200', 80)
    r = c.slb.hm.get("hfoobar")
    print("LIBRARY RESPONSE = %s" % r)
    try:
        c.slb.hm.create("hfoobar", c.slb.hm.HTTP, 5, 5, 5, 'GET', '/', '200',
                        80)
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.hm.update("hfoobar", c.slb.hm.HTTP, 10, 10, 10, 'GET', '/', '200', 80)
    try:
        c.slb.hm.update("hnfoobar", c.slb.hm.HTTP, 10, 10, 10, 'GET', '/', '200', 80)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.hm.delete("hfoobar")
    c.slb.hm.delete("hfoobar")
    try:
        c.slb.hm.get("hfoobar")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()

    c.slb.hm.delete("hm2")
    c.slb.hm.create("hm2", c.slb.hm.ICMP, 5, 5, 5)
    r = c.slb.hm.get("hm2")

    c.slb.hm.delete("hm4")
    c.slb.hm.create("hm4", c.slb.hm.TCP, 5, 5, 5, port=25)
    r = c.slb.hm.get("hm4")

    c.slb.hm.delete("hm3")
    c.slb.hm.create("hm3", c.slb.hm.HTTPS, 5, 5, 5, 'GET', '/', '200', 443)
    r = c.slb.hm.get("hm3")

    # Test that alternate port is used.
    c.slb.hm.create("hport", c.slb.hm.HTTP, 5, 5, 5, 'GET', '/', '200', 81)
    c.slb.hm.get("hport")
    c.slb.hm.delete("hport")

    print("=============================================================")
    print("")
    print("Member Create")
    c.slb.service_group.member.delete("pfoobar", "foobar", 80)
    c.slb.service_group.member.create("pfoobar", "foobar", 80)
    try:
        c.slb.service_group.member.create("pfoobar", "foobar", 80)
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    try:
        oper = c.slb.service_group.oper("vfoobar")
        print("OPERATIONAL STATS (SERVICE GROUP):")
        print(oper)
    except Exception:
        pass

    print("Member get_oper")
    oper = c.slb.service_group.member.get_oper("pfoobar", "foobar", 80)
    print(oper)

    c.slb.service_group.member.update("pfoobar", "foobar", 80,
                                      c.slb.DOWN)
    try:
        c.slb.service_group.member.update("pfoobar", "nfoobar", 80)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    try:
        c.slb.service_group.member.update("pnfoobar", "foobar", 80)
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.service_group.member.delete("pfoobar", "foobar", 80)
    c.slb.service_group.member.delete("pfoobar", "foobar", 80)

    print("=============================================================")
    print("")
    print("Source Ip Persistence")
    c.slb.template.src_ip_persistence.delete("sip1")
    c.slb.template.src_ip_persistence.create("sip1")
    try:
        c.slb.template.src_ip_persistence.create("sip1")
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    r = c.slb.template.src_ip_persistence.get("sip1")
    print("LIBRARY RESPONSE = %s" % r)
    c.slb.template.src_ip_persistence.exists("sip1")
    c.slb.template.src_ip_persistence.delete("sip1")
    c.slb.template.src_ip_persistence.delete("sip1")
    try:
        c.slb.template.src_ip_persistence.get("sip1")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.template.src_ip_persistence.create("sip1")

    print("=============================================================")
    print("")
    print("Http Cookie Persistence")
    c.slb.template.cookie_persistence.delete("cp1")
    c.slb.template.cookie_persistence.create("cp1")
    try:
        c.slb.template.cookie_persistence.create("cp1")
    except acos_client.errors.Exists:
        print("got already exists error, good")
    else:
        raise Nope()
    c.slb.template.cookie_persistence.get("cp1")
    c.slb.template.cookie_persistence.exists("cp1")
    c.slb.template.cookie_persistence.delete("cp1")
    c.slb.template.cookie_persistence.delete("cp1")
    try:
        c.slb.template.cookie_persistence.get("cp1")
    except acos_client.errors.NotFound:
        print("got not found, good")
    else:
        raise Nope()
    c.slb.template.cookie_persistence.create("cp1")

    print("=============================================================")
    print("")
    print("Vip with pers")
    c.slb.virtual_server.delete("vip2")
    c.slb.virtual_server.create("vip2", pmap['vip2'])
    c.slb.virtual_server.vport.create(
        "vip2", "vip2_vport2",
        protocol=c.slb.virtual_server.vport.HTTPS,
        port=444,
        service_group_name='pfoobar',
        s_pers_name='sip1')
    c.slb.virtual_server.vport.create(
        "vip2", "vip2_vport3",
        protocol=c.slb.virtual_server.vport.HTTPS,
        port=445,
        service_group_name='pfoobar',
        c_pers_name='cp1')

    print("=============================================================")
    print("")
    print("vip for vport-only tests")
    c.slb.virtual_server.delete("vip4")
    c.slb.virtual_server.create("vip4", pmap['vip4'])

    print("=============================================================")
    print("")
    print("vport with auto source nat pool")
    c.slb.virtual_server.vport.create(
        "vip4", "vip4-autosnat",
        protocol=c.slb.virtual_server.vport.HTTP,
        port=80,
        service_group_name='pfoobar',
        autosnat=True)
    c.slb.virtual_server.vport.create(
        "vip4", "vip4-noautosnat",
        protocol=c.slb.virtual_server.vport.HTTP,
        port=81,
        service_group_name='pfoobar',
        autosnat=False)

    print("=============================================================")
    print("")
    print("vport with ip in ip")
    try:
        c.slb.virtual_server.vport.create(
            "vip4", "vip4-ipinip",
            protocol=c.slb.virtual_server.vport.HTTP,
            port=90,
            service_group_name='pfoobar',
            ipinip=True)
    except Exception:
        pass
    c.slb.virtual_server.vport.create(
        "vip4", "vip4-noipinip",
        protocol=c.slb.virtual_server.vport.HTTP,
        port=91,
        service_group_name='pfoobar',
        ipinip=False)

    print("=============================================================")
    print("sflow settings")
    print("=============================================================")
    print("")

    sflow_ip = "10.48.9.50"
    sflow_port = 6343

    try:
        c.sflow.setting.create(60, True, 60, 60)
    except NotImplementedError:
        print("sflow not implemented in 2.1")

    try:
        c.sflow.collector.ip.create(sflow_ip, sflow_port)
    except acos_client.errors.Exists:
        pass
    except Exception:
        print("Failed setting sflow collector")

    try:
        c.sflow.polling.create(True)
    except Exception:
        print("Failed setting sflow polling")

    print("=============================================================")
    print("")
    print("License Manager")
    lm_host = {"ip": "10.200.0.1", "port": 443}
    try:
        print("... Create")
        c.license_manager.create([lm_host])
    except NotImplementedError:
        print("License Manager not implemented in %s" % ARGS.axapi_version)
    except acos_client.errors.FeatureNotSupported:
        print("License Manager is not supported on this platform")

    try:
        print("... Get")
        c.license_manager.get()
    except NotImplementedError:
        print("License Manager not implemented in %s" % ARGS.axapi_version)
    except acos_client.errors.FeatureNotSupported:
        print("License Manager is not supported on this platform")

    lm_host["ip"] = "10.200.0.2"
    try:
        print("... Update")
        c.license_manager.update([lm_host])
    except NotImplementedError:
        print("License Manager not implemented in %s" % ARGS.axapi_version)
    except acos_client.errors.FeatureNotSupported:
        print("License Manager is not supported on this platform")

    if float(ARGS.axapi_version) >= 3.0:
        print("=============================================================")
        print("")
        print("slb.common")
        print("dsr_health_check")
        try:
            c.slb.common.create(dsr_health_check_enable=1)
        except NotImplementedError:
            print("DSR Health Check not implemented in %s" % ARGS.axapi_version)
        print("... Get updated")

    print("=============================================================")
    print("")
    print("Interface Tests")
    print("")
    print("=============================================================")

    fake_static = "10.200.13.142"
    fake_mask = "255.255.255.0"

    if float(ARGS.axapi_version) >= 3.0:
        eth_ifs = c.interface.ethernet.get()
        mgmt_if = c.interface.management.get()

        print("Ethernet Interfaces:\r\n{0}".format(eth_ifs))
        print("Manage Interfaces:\r\n{0}".format(mgmt_if))
        eth1 = c.interface.ethernet.get(1)
        print("Ethernet Interface 1:\r\n{0}".format(eth1))
        print("Updating interface 1 with DHCP...")
        c.interface.ethernet.update(1, enable=False)
        print("Updating interface 1 with fake IP...")
        try:
            c.interface.ethernet.update(1, dhcp=False, ip_address="",
                                        ip_netmask="", enable=False)
            c.interface.ethernet.update(1, dhcp=False, ip_address="10.200.0.1",
                                        ip_netmask="255.255.255.0", enable=True)
        except acos_client.errors.ACOSException:
            print("Could not update interface")

        c.interface.ethernet.get(1)

        eth1_dhcp = c.interface.ethernet.get(1)
        print("Updated interface 1 DHCP: {0}".format(eth1_dhcp))
    elif float(ARGS.axapi_version) == 2.1:

        eth_ifs = c.interface.ethernet.get_list()
        mgmt_if = c.interface.management.get()

        print("INITIAL Ethernet Interfaces:\r\n{0}".format(eth_ifs))
        print("INITIAL Management Interfaces:\r\n{0}".format(mgmt_if))
        eth1 = c.interface.ethernet.get(1)
        print("INITIAL Ethernet Interface 1:\r\n{0}".format(eth1))
        print("Updating interface 1 with DHCP...")
        c.interface.ethernet.update(1, enable=False)
        c.interface.ethernet.update(1, dhcp=True, enable=False)
        eth1_updated = c.interface.ethernet.get(1)

        eth1_dhcp_ip = eth1_updated["interface"]["ipv4_list"][0]["ipv4_address"]
        eth1_dhcp_mask = eth1_updated["interface"]["ipv4_list"][0]["ipv4_mask"]
        eth1_dhcp_ip = fake_static
        eth1_dhcp_mask = fake_mask

        print("DHCP INFO: IP: {0}  NETMASK:{1}".format(eth1_dhcp_ip, eth1_dhcp_mask))
        print("Interface post-DHCP update: {0}".format(eth1_updated))
        print("Updating interface 1 with fake IP...")
        try:
            c.interface.ethernet.update(1, dhcp=False, ip_address=fake_static,
                                        ip_netmask=fake_mask, enable=False)
            c.interface.ethernet.update(1, dhcp=False, ip_address=fake_static,
                                        ip_netmask=fake_mask, enable=True)
        except acos_client.errors.ACOSException:
            print("Could not update interface")
        eth1_static = c.interface.ethernet.get(1)
        print("Updated interface 1 static: {0}".format(eth1_static))
        if fake_static not in [x["ipv4_address"] for x in eth1_static["interface"]["ipv4_list"]]:
            raise Nope("Interface IPs don't match.")

        try:
            c.interface.ethernet.create(1, dhcp=True, enable=True)
            eth1_dhcp = c.interface.ethernet.get(1)
            print("Updated interface 1 DHCP: {0}".format(eth1_dhcp))
        except acos_client.errors.ACOSException:
            print("Could not update interface")

    print("=============================================================")
    print("")
    print("About half the time, delete the partition!")

    if int(random.random() * 2):
        try:
            c.system.partition.delete(partition)
        except acos_client.errors.InUse:
            if ARGS.axapi_version == "2.1":
                print("AXAPI 2.1 can't delete partitions until admin session is cleared.")
            else:
                raise

        try:
            c.system.partition.delete(partition)
        except acos_client.errors.InUse:
            if ARGS.axapi_version == "2.1":
                print("AXAPI 2.1 can't delete partitions until admin session is cleared.")
            else:
                raise
        except acos_client.errors.NotFound:
            pass

    c.session.close()

    if float(ARGS.axapi_version) >= 3.0:
        c.file.ssl_key.delete(c_key)
        c.file.ssl_key.delete(s_key)
        c.file.ssl_cert.delete(c_cert)
        c.file.ssl_cert.delete(s_cert)

    print("=============================================================")
    print("t.py completed successfully!")
    print("=============================================================")


def main():
    ax = {
        'host': ARGS.host,
        'port': ARGS.port,
        'protocol': ARGS.protocol,
        'user': ARGS.user,
        'password': ARGS.password,
        'axapi': ARGS.axapi_version,
    }
    z = [ARGS.partition]
    if z[0] == 'all':
        z = partitions
    for k in z:
        v = partition_map[k]
        partition = v['name']
        try:
            run_all(ax, partition, v)
        except Exception as e:
            traceback.print_exc()
            print(e)
            sys.exit(1)


if __name__ == '__main__':
    main()
    sys.exit(0)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from acos_client import client
import acos_client.errors as acos_errors
import responses


HOSTNAME = 'fake_a10'
BASE_URL = "https://{}:443/services/rest/v2.1/?format=json&method=".format(HOSTNAME)
AUTH_URL = "{}authenticate".format(BASE_URL)
CREATE_URL = '{}slb.server.create&session_id={}'.format(BASE_URL, 'foobar')
DELETE_URL = '{}slb.server.delete&session_id={}'.format(BASE_URL, 'foobar')
SEARCH_URL = '{}slb.server.search&session_id={}'.format(BASE_URL, 'foobar')


class TestServer(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')

    @responses.activate
    def test_server_create(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        resp = self.client.slb.server.create('test', '192.168.2.254')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_server_create_already_exists(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 402653200, "msg": " Name or IP address already exists."}}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.Exists):
            self.client.slb.server.create('test', '192.168.2.254')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_server_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.server.delete('test')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)

    @responses.activate
    def test_server_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 67174402, "msg": " No such Server"}}
        }
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.server.delete('test')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)

    @responses.activate
    def test_server_search(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'server': {
                'status': 1, 'conn_resume': 0, 'weight': 1, 'conn_limit': 8000000, 'host': '192.168.2.254',
                'spoofing_cache': 0, 'port_list': [], 'gslb_external_address': '0.0.0.0', 'slow_start': 0,
                'name': 's1', 'health_monitor': '(default)', 'extended_stats': 0, 'template': 'default',
                'stats_data': 1, 'conn_limit_log': 0
            }
        }
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        resp = self.client.slb.server.get('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)

    @responses.activate
    def test_server_search_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 67174402, "msg": " No such Server"}}
        }
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.NotFound):
            self.client.slb.server.get('test')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)


class TestIPv6Server(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')

    @responses.activate
    def test_server_create(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        resp = self.client.slb.server.create('test', '2001:baad:deed:bead:daab:daad:cead:100e')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_server_create_already_exists(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 402653200, "msg": " Name or IP address already exists."}}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.Exists):
            self.client.slb.server.create('test', '2001:baad:deed:bead:daab:daad:cead:100e')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @responses.activate
    def test_server_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'response': {'status': 'OK'}
        }
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.server.delete('test')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)

    @responses.activate
    def test_server_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 67174402, "msg": " No such Server"}}
        }
        responses.add(responses.POST, DELETE_URL, json=json_response, status=200)

        resp = self.client.slb.server.delete('test')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, DELETE_URL)

    @responses.activate
    def test_server_search(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            'server': {
                'status': 1, 'conn_resume': 0, 'weight': 1, 'conn_limit': 8000000,
                'host': '2001:baad:deed:bead:daab:daad:cead:100e',
                'spoofing_cache': 0, 'port_list': [], 'gslb_external_address': '0.0.0.0', 'slow_start': 0,
                'name': 's1', 'health_monitor': '(default)', 'extended_stats': 0, 'template': 'default',
                'stats_data': 1, 'conn_limit_log': 0
            }
        }
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        resp = self.client.slb.server.get('test')

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)

    @responses.activate
    def test_server_search_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 67174402, "msg": " No such Server"}}
        }
        responses.add(responses.POST, SEARCH_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.NotFound):
            self.client.slb.server.get('test')

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, SEARCH_URL)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client import errors as ae
from acos_client.v30 import base


class VirtualPort(base.BaseV30):

    # Protocols
    TCP = "tcp"
    UDP = "udp"
    OTHERS = "others"
    DIAMETER = "diameter"
    DNS_TCP = "dns-tcp"
    DNS_UDP = "dns-udp"
    FAST_HTTP = "fast-http"
    FIX = "fix"
    FTP = "ftp"
    FTP_PROXY = "ftp-proxy"
    HTTP = "http"
    HTTPS = "https"
    MLB = "mlb"
    MMS = "mms"
    MYSQL = "mysql"
    MSSQL = "mssql"
    RADIUS = "radius"
    RTSP = "rtsp"
    SIP = "sip"
    SIP_TCP = "sip-tcp"
    SIPS = "sips"
    SMPP_TCP = "smpp-tcp"
    SPDY = "spdy"
    SPDYS = "spdys"
    SMTP = "smtp"
    SSL_PROXY = "ssl-proxy"
    TCP_PROXY = "tcp-proxy"
    TFTP = "tftp"
    GENERIC_PROXY = "tcp-proxy"

    url_server_tmpl = '/slb/virtual-server/{name}/port/'
    url_port_tmpl = '{port_number}+{protocol}'

    def format_vport_url(self, port_number, protocol):
        if protocol == 'TERMINATED_HTTPS':
            protocol = "https"
        return self.url_port_tmpl.format(port_number=port_number, protocol=protocol)

    def all(self, virtual_server_name, **kwargs):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        return self._get(url, **kwargs)

    def get(self, virtual_server_name, name, protocol, port):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        url += self.format_vport_url(port_number=port, protocol=protocol)
        return self._get(url)

    def _set(
        self,
        virtual_server_name,
        name,
        protocol,
        protocol_port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=0,
        no_dest_nat=None,
        autosnat=None,
        ipinip=None,
        source_nat_pool=None,
        ha_conn_mirror=None,
        use_rcv_hop=None,
        conn_limit=None,
        virtual_port_templates=None,
        tcp_template=None,
        udp_template=None,
        exclude_minimize=None,
        update=False,
        template_server_ssl=None,
        template_client_ssl=None,
        sampling_enable=None,
        aflex_scripts=None,
        **kwargs
    ):
        exclude_minimize = [] if exclude_minimize is None else exclude_minimize
        params = {
            "port": self.minimal_dict({
                "name": name,
                "service-group": service_group_name,
                "protocol": protocol,
                "port-number": int(protocol_port),
                "template-persist-source-ip": s_pers_name,
                "template-persist-cookie": c_pers_name,
                "extended-stats": status
            }, exclude=exclude_minimize
            )
        }
        if virtual_port_templates:
            virtual_port_templates = {k: v for k, v in virtual_port_templates.items() if v}

            if virtual_port_templates.get('template-virtual-port'):
                params['port']['template-virtual-port'] = virtual_port_templates['template-virtual-port']
            elif virtual_port_templates.get('template-virtual-port-shared'):
                params['port']['template-virtual-port-shared'] = virtual_port_templates['template-virtual-port-shared']
                params['port']['shared-partition-virtual-port-template'] = True

            if protocol in ['http', 'https']:
                if virtual_port_templates.get('template-http'):
                    params['port']['template-http'] = virtual_port_templates['template-http']
                elif virtual_port_templates.get('template-http-shared'):
                    params['port']['template-http-shared'] = virtual_port_templates['template-http-shared']
                    params['port']['shared-partition-http-template'] = True
            else:
                if virtual_port_templates.get('template-tcp'):
                    params['port']['template-tcp'] = virtual_port_templates['template-tcp']
                elif virtual_port_templates.get('template-tcp-shared'):
                    params['port']['template-tcp-shared'] = virtual_port_templates['template-tcp-shared']
                    params['port']['shared-partition-tcp'] = True

            if virtual_port_templates.get('template-policy'):
                params['port']['template-policy'] = virtual_port_templates['template-policy']
            elif virtual_port_templates.get('template-policy-shared'):
                params['port']['template-policy-shared'] = virtual_port_templates['template-policy-shared']
                params['port']['shared-partition-policy-template'] = True

        if autosnat is not None:
            params['port']['auto'] = int(autosnat)
        if ipinip is not None:
            params['port']['ipinip'] = int(ipinip)
        if use_rcv_hop is not None:
            params['port']['use-rcv-hop-for-resp'] = int(use_rcv_hop)

        if source_nat_pool and len(source_nat_pool) > 0:
            params['port']['pool'] = source_nat_pool
        if tcp_template:
            params['port']['tcp_template'] = tcp_template
        if udp_template:
            params['port']['udp_template'] = udp_template

        if template_server_ssl:
            params['port']['template-server-ssl'] = template_server_ssl
        if template_client_ssl:
            params['port']['template-client-ssl'] = template_client_ssl

        if sampling_enable is not None:
            self._set_sampling_enable(sampling_enable, params)

        if no_dest_nat is not None:
            params["port"]["no-dest-nat"] = 1 if no_dest_nat else 0
        if ha_conn_mirror is not None:
            if (protocol == self.TCP or protocol == self.UDP):
                params["port"]["ha-conn-mirror"] = 1 if ha_conn_mirror else 0
        if conn_limit is not None:
            params["port"]["conn-limit"] = conn_limit

        url = self.url_server_tmpl.format(name=virtual_server_name)

        if aflex_scripts is not None:
            params['port']['aflex-scripts'] = aflex_scripts

        if update:
            url += self.format_vport_url(port_number=protocol_port, protocol=protocol)
        return url, params, kwargs

    def create(
        self,
        virtual_server_name,
        name,
        protocol,
        protocol_port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=1,
        autosnat=None,
        ipinip=None,
        no_dest_nat=None,
        source_nat_pool=None,
        ha_conn_mirror=None,
        use_rcv_hop=None,
        conn_limit=None,
        virtual_port_templates=None,
        tcp_template=None,
        udp_template=None,
        max_retries=None,
        timeout=None,
        template_server_ssl=None,
        template_client_ssl=None,
        sampling_enable=None,
        aflex_scripts=None,
        **kwargs
    ):

        # backward compatiable for a10-neutron-lbaas
        if aflex_scripts is None and 'aflex-scripts' in kwargs:
            aflex_scripts = kwargs.pop('aflex-scripts')

        url, params, kwargs = self._set(
            virtual_server_name,
            name,
            protocol,
            protocol_port,
            service_group_name,
            s_pers_name,
            c_pers_name,
            status,
            autosnat=autosnat,
            ipinip=ipinip,
            no_dest_nat=no_dest_nat,
            source_nat_pool=source_nat_pool,
            ha_conn_mirror=ha_conn_mirror,
            use_rcv_hop=use_rcv_hop,
            conn_limit=conn_limit,
            virtual_port_templates=virtual_port_templates,
            tcp_template=tcp_template,
            udp_template=udp_template,
            template_server_ssl=template_server_ssl,
            template_client_ssl=template_client_ssl,
            sampling_enable=sampling_enable,
            aflex_scripts=aflex_scripts,
            **kwargs
        )

        return self._post(url, params, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def _update(
        self,
        virtual_server_name,
        name,
        protocol,
        protocol_port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=1,
        autosnat=None,
        ipinip=None,
        no_dest_nat=None,
        source_nat_pool=None,
        ha_conn_mirror=None,
        use_rcv_hop=None,
        conn_limit=None,
        virtual_port_templates=None,
        tcp_template=None,
        udp_template=None,
        template_server_ssl=None,
        template_client_ssl=None,
        sampling_enable=None,
        aflex_scripts=None,
        **kwargs
    ):

        vp = self.get(virtual_server_name, name, protocol, protocol_port)
        if vp is None:
            raise ae.NotFound()

        exclude = ['template-persist-source-ip', 'template-persist-cookie']

        try:
            url, params, kwargs = self._set(
                virtual_server_name,
                name,
                protocol,
                protocol_port,
                service_group_name,
                s_pers_name,
                c_pers_name,
                status,
                autosnat=autosnat,
                ipinip=ipinip,
                no_dest_nat=no_dest_nat,
                source_nat_pool=source_nat_pool,
                ha_conn_mirror=ha_conn_mirror,
                use_rcv_hop=use_rcv_hop,
                conn_limit=conn_limit,
                virtual_port_templates=virtual_port_templates,
                tcp_template=tcp_template,
                udp_template=udp_template,
                exclude_minimize=exclude,
                update=True,
                template_server_ssl=template_server_ssl,
                template_client_ssl=template_client_ssl,
                sampling_enable=sampling_enable,
                aflex_scripts=aflex_scripts,
                **kwargs
            )
        except ae.AxapiJsonFormatError:
            url, params, kwargs = self._set(
                virtual_server_name,
                name,
                protocol,
                protocol_port,
                service_group_name,
                s_pers_name,
                c_pers_name,
                status,
                autosnat=autosnat,
                ipinip=ipinip,
                no_dest_nat=no_dest_nat,
                source_nat_pool=source_nat_pool,
                ha_conn_mirror=ha_conn_mirror,
                use_rcv_hop=use_rcv_hop,
                conn_limit=conn_limit,
                virtual_port_templates=virtual_port_templates,
                tcp_template=tcp_template,
                udp_template=udp_template,
                exclude_minimize=exclude,
                update=True,
                template_server_ssl=template_server_ssl,
                template_client_ssl=template_client_ssl,
                sampling_enable=sampling_enable,
                aflex_scripts=aflex_scripts,
                **kwargs
            )
        return url, params, kwargs

    def update(
        self,
        virtual_server_name,
        name,
        protocol,
        protocol_port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=1,
        autosnat=None,
        ipinip=None,
        no_dest_nat=None,
        source_nat_pool=None,
        ha_conn_mirror=None,
        use_rcv_hop=None,
        conn_limit=None,
        virtual_port_templates=None,
        tcp_template=None,
        udp_template=None,
        max_retries=None,
        timeout=None,
        template_server_ssl=None,
        template_client_ssl=None,
        sampling_enable=None,
        aflex_scripts=None,
        **kwargs
    ):

        # backward compatiable for a10-neutron-lbaas
        if aflex_scripts is None and 'aflex-scripts' in kwargs:
            aflex_scripts = kwargs.pop('aflex-scripts')

        url, params, kwargs = self._update(
            virtual_server_name,
            name,
            protocol,
            protocol_port,
            service_group_name,
            s_pers_name=s_pers_name,
            c_pers_name=c_pers_name,
            status=status,
            autosnat=autosnat,
            ipinip=ipinip,
            no_dest_nat=no_dest_nat,
            source_nat_pool=source_nat_pool,
            ha_conn_mirror=ha_conn_mirror,
            use_rcv_hop=use_rcv_hop,
            conn_limit=conn_limit,
            virtual_port_templates=virtual_port_templates,
            tcp_template=tcp_template,
            udp_template=udp_template,
            template_server_ssl=template_server_ssl,
            template_client_ssl=template_client_ssl,
            sampling_enable=sampling_enable,
            aflex_scripts=aflex_scripts,
            **kwargs)
        return self._post(url, params, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def replace(
        self,
        virtual_server_name,
        name,
        protocol,
        protocol_port,
        service_group_name,
        s_pers_name=None,
        c_pers_name=None,
        status=1,
        autosnat=None,
        ipinip=None,
        no_dest_nat=None,
        source_nat_pool=None,
        ha_conn_mirror=None,
        use_rcv_hop=None,
        conn_limit=None,
        virtual_port_templates=None,
        tcp_template=None,
        udp_template=None,
        max_retries=None,
        timeout=None,
        template_server_ssl=None,
        template_client_ssl=None,
        sampling_enable=None,
        aflex_scripts=None,
        **kwargs
    ):

        # backward compatiable for a10-neutron-lbaas
        if aflex_scripts is None and 'aflex-scripts' in kwargs:
            aflex_scripts = kwargs.pop('aflex-scripts')

        url, params, kwargs = self._update(
            virtual_server_name,
            name,
            protocol,
            protocol_port,
            service_group_name,
            s_pers_name=s_pers_name,
            c_pers_name=c_pers_name,
            status=status,
            autosnat=autosnat,
            ipinip=ipinip,
            no_dest_nat=no_dest_nat,
            source_nat_pool=source_nat_pool,
            ha_conn_mirror=ha_conn_mirror,
            use_rcv_hop=use_rcv_hop,
            conn_limit=conn_limit,
            virtual_port_templates=virtual_port_templates,
            tcp_template=tcp_template,
            udp_template=udp_template,
            template_server_ssl=template_server_ssl,
            template_client_ssl=template_client_ssl,
            sampling_enable=sampling_enable,
            aflex_scripts=aflex_scripts,
            **kwargs)
        return self._put(url, params, max_retries=max_retries, timeout=timeout, axapi_args=kwargs)

    def delete(self, virtual_server_name, name, protocol, port):
        url = self.url_server_tmpl.format(name=virtual_server_name)
        url += self.format_vport_url(port_number=port, protocol=protocol)
        return self._delete(url)

    def _set_sampling_enable(self, sample_list, dest_obj):
        dest_array = []
        for x in sample_list:
            entry = {"counters1": x}
            dest_array.append(entry)

        dest_obj["port"]["sampling-enable"] = dest_array



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base
from acos_client.v30.slb.port import Port


class Server(base.BaseV30):

    url_prefix = '/slb/server/'

    def get(self, name, max_retries=None, timeout=None, **kwargs):
        return self._get(self.url_prefix + name, max_retries=max_retries, timeout=timeout,
                         axapi_args=kwargs)

    def get_all(self):
        return self._get(self.url_prefix)

    def _set(self, name, ip_address, status=1, server_templates=None, port_list=None,
             conn_resume=None, conn_limit=None, health_check=None, **kwargs):
        params = {
            "server": {
                "name": name,
                "action": 'enable' if status else 'disable',
                "conn-resume": conn_resume,
                "conn-limit": conn_limit,
                "health-check": health_check,
            }
        }

        if port_list:
            params['server']['port-list'] = port_list

        if self._is_ipv6(ip_address):
            params['server']['server-ipv6-addr'] = ip_address
        else:
            params['server']['host'] = ip_address

        if server_templates:
            server_templates = {k: v for k, v in server_templates.items() if v}
            params['server']['template-server'] = server_templates.get('template-server')

        return params

    def create(self, name, ip_address, status=1, server_templates=None,
               port_list=None, max_retries=None, timeout=None,
               conn_resume=None, conn_limit=None, health_check=None, **kwargs):
        params = self._set(name, ip_address, status=status, port_list=port_list,
                           conn_resume=conn_resume, conn_limit=conn_limit, health_check=health_check,
                           server_templates=server_templates, **kwargs)
        return self._post(self.url_prefix, params, max_retries=max_retries, timeout=timeout,
                          axapi_args=kwargs)

    def update(self, name, ip_address, status=1, server_templates=None,
               port_list=None, max_retries=None, timeout=None,
               conn_resume=None, conn_limit=None, health_check=None, **kwargs):
        params = self._set(name, ip_address, status=status, port_list=port_list,
                           conn_resume=conn_resume, conn_limit=conn_limit, health_check=health_check,
                           server_templates=server_templates, **kwargs)
        return self._post(self.url_prefix + name, params, max_retries=max_retries, timeout=timeout,
                          axapi_args=kwargs)

    def replace(self, name, ip_address, status=1, server_templates=None,
                port_list=None, max_retries=None, timeout=None,
                conn_resume=None, conn_limit=None, health_check=None, **kwargs):
        params = self._set(name, ip_address, status=status, port_list=port_list,
                           conn_resume=conn_resume, conn_limit=conn_limit, health_check=health_check,
                           server_templates=server_templates, **kwargs)
        return self._put(self.url_prefix + name, params, max_retries=max_retries, timeout=timeout,
                         axapi_args=kwargs)

    def delete(self, name):
        return self._delete(self.url_prefix + name)

    @property
    def port(self):
        return Port(self.client)



# Copyright 2014,  Jeff Buttars,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base


class OverlayOptions(base.BaseV30):
    url_prefix = "/overlay-tunnel/options"

    def get(self, *args, **kwargs):
        return self._get(self.url_prefix, **kwargs)

    def update(self, gateway_mac, ip_dscp_preserve,
               nvgre_disable_flow_id,
               nvgre_key_mode_lower24,
               tcp_mss_adjust_disable,
               uuid,
               vxlan_dest_port,
               **kwargs):

        options = {}

        if gateway_mac:
            options["gateway-mac"] = gateway_mac

        if ip_dscp_preserve:
            options["ip-dscp-preserve"] = ip_dscp_preserve

        if nvgre_disable_flow_id:
            options["nvgre-disable-flow-id"] = nvgre_disable_flow_id

        if nvgre_key_mode_lower24:
            options["nvgre-key-mode-lower24"] = nvgre_key_mode_lower24

        if tcp_mss_adjust_disable:
            options["tcp-mss-adjust-disable"] = tcp_mss_adjust_disable

        if uuid:
            options["uuid"] = uuid

        if vxlan_dest_port:
            options["vxlan-dest-port"] = vxlan_dest_port

        payload = {
            "options": options
        }

        return self._post(self.url_prefix + "/options", payload, **kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest
    from unittest import mock
except ImportError:
    import mock
    import unittest2 as unittest

from acos_client import client
import acos_client.errors as acos_errors
import json
import responses


HOSTNAME = 'fake_a10'
BASE_URL = 'https://{}:443/axapi/v3'.format(HOSTNAME)
AUTH_URL = '{}/auth'.format(BASE_URL)
VSERVER_NAME = 'test'
CREATE_URL = '{}/slb/virtual-server/{}/port/'.format(BASE_URL, VSERVER_NAME)
OBJECT_URL = '{}/slb/virtual-server/{}/port/80+http'.format(BASE_URL, VSERVER_NAME)
OBJECT_TCP_URL = '{}/slb/virtual-server/{}/port/80+tcp'.format(BASE_URL, VSERVER_NAME)
ALL_URL = '{}/slb/virtual-server/{}/port/'.format(BASE_URL, VSERVER_NAME)
OK_RESP = {'response': {'status': 'OK'}}


class TestVirtualPort(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '30', 'fake_username', 'fake_password')

    @responses.activate
    def test_virtual_port_create_no_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
        params = {
            'port':
            {
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'port-number': 80,
                'protocol': 'http',
                'service-group': 'pool1'
            }
        }

        resp = self.client.slb.virtual_server.vport.create(
            VSERVER_NAME, 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, protocol_port='80',
            service_group_name='pool1'
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_create_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'ipinip': 1,
                'name': 'test1_VPORT',
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'http',
                'service-group': 'pool1',
                'no-dest-nat': 1,
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'use-rcv-hop-for-resp': 1
            }
        }

        resp = self.client.slb.virtual_server.vport.create(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            status=1,
            autosnat=True,
            ipinip=True,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            use_rcv_hop=True,
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_tcp_virtual_port_create_with_params(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'ipinip': 1,
                'name': 'test1_VPORT',
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'tcp',
                'service-group': 'pool1',
                'ha-conn-mirror': 1,
                'no-dest-nat': 1,
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'use-rcv-hop-for-resp': 1
            }
        }

        resp = self.client.slb.virtual_server.vport.create(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.TCP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            status=1,
            autosnat=True,
            ipinip=True,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            use_rcv_hop=True,
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_create_with_kwargs(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'ipinip': 1,
                'name': 'test1_VPORT',
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'http',
                'service-group': 'pool1',
                'no-dest-nat': 1,
                'conn-limit': 400,
                'use-rcv-hop-for-resp': 1
            }
        }
        kwargs = {
            'conn_limit': 400,
        }

        resp = self.client.slb.virtual_server.vport.create(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            ha_conn_mirror=1,
            no_dest_nat=1,
            status=1,
            autosnat=True,
            ipinip=True,
            source_nat_pool="test_nat_pool",
            use_rcv_hop=True,
            **kwargs
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_tcp_virtual_port_create_with_kwargs(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'ipinip': 1,
                'name': 'test1_VPORT',
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'tcp',
                'service-group': 'pool1',
                'ha-conn-mirror': 1,
                'no-dest-nat': 1,
                'conn-limit': 400,
                'use-rcv-hop-for-resp': 1
            }
        }
        kwargs = {
            'conn_limit': 400,
        }

        resp = self.client.slb.virtual_server.vport.create(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.TCP,
            protocol_port='80',
            service_group_name='pool1',
            ha_conn_mirror=1,
            no_dest_nat=1,
            status=1,
            autosnat=True,
            ipinip=True,
            source_nat_pool="test_nat_pool",
            use_rcv_hop=True,
            **kwargs
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_create_already_exists(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 1406, "msg": "The virtual port already exists."}}
        }
        responses.add(responses.POST, CREATE_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.ACOSException):
            self.client.slb.virtual_server.vport.create(
                VSERVER_NAME, 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, protocol_port='80',
                service_group_name='pool1'
            )

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_update_no_params(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)
        params = {
            "port":
            {
                "name": "test1_VPORT",
                "service-group": "pool1",
                "protocol": "http",
                "port-number": 80,
                "template-persist-source-ip": None,
                "template-persist-cookie": None,
                "extended-stats": 1
            }
        }

        resp = self.client.slb.virtual_server.vport.update(
            VSERVER_NAME, 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, protocol_port='80',
            service_group_name='pool1'
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_create_with_templates(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
        protocol = self.client.slb.virtual_server.vport.HTTP
        if protocol.lower() == 'http':
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'ipinip': 1,
                    'name': 'test1_VPORT',
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp',
                    'template-policy': 'template_pl'
                }
            }
        else:
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'ipinip': 1,
                    'name': 'test1_VPORT',
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp',
                    'template-tcp': 'template_tcp',
                    'template-policy': 'template_pl'
                }
            }

        resp = self.client.slb.virtual_server.vport.create(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            virtual_port_templates={
                'template-virtual-port': 'template_vp',
                'template-tcp': 'template_tcp',
                'template-policy': 'template_pl',
            },
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_create_with_partial_templates(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, CREATE_URL, json=OK_RESP, status=200)
        protocol = self.client.slb.virtual_server.vport.HTTP
        if protocol.lower() == 'http':
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'ipinip': 1,
                    'name': 'test1_VPORT',
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp'
                }
            }
        else:
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'ipinip': 1,
                    'name': 'test1_VPORT',
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp',
                    'template-tcp': None,
                    'template-policy': None,
                }
            }

        resp = self.client.slb.virtual_server.vport.create(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            virtual_port_templates={
                'template-virtual-port': 'template_vp'
            },
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, CREATE_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_update_with_params(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'ipinip': 1,
                'no-dest-nat': 1,
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'http',
                'service-group': 'pool1',
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'use-rcv-hop-for-resp': 1,
            }
        }

        resp = self.client.slb.virtual_server.vport.update(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            use_rcv_hop=True,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_tcp_virtual_port_update_with_params(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.POST, OBJECT_TCP_URL, json=json_response, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'ipinip': 1,
                'no-dest-nat': 1,
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'tcp',
                'service-group': 'pool1',
                'ha-conn-mirror': 1,
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'use-rcv-hop-for-resp': 1,
            }
        }

        resp = self.client.slb.virtual_server.vport.update(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.TCP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            use_rcv_hop=True,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, OBJECT_TCP_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_update_with_kwargs(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'ipinip': 1,
                'no-dest-nat': 1,
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'http',
                'service-group': 'pool1',
                'conn-limit': 400,
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'use-rcv-hop-for-resp': 1,
            }
        }
        kwargs = {
            'conn_limit': 400
        }

        resp = self.client.slb.virtual_server.vport.update(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=1,
            source_nat_pool="test_nat_pool",
            use_rcv_hop=True,
            **kwargs
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_update_with_templates(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.POST, OBJECT_URL, json=json_response, status=200)
        protocol = self.client.slb.virtual_server.vport.HTTP
        if protocol.lower() == 'http':
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'name': 'test1_VPORT',
                    'ipinip': 1,
                    'no-dest-nat': 0,
                    'use-rcv-hop-for-resp': 0,
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'conn-limit': 50000,
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp'
                }
            }
        else:
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'name': 'test1_VPORT',
                    'ipinip': 1,
                    'no-dest-nat': 0,
                    'use-rcv-hop-for-resp': 0,
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'ha-conn-mirror': 1,
                    'conn-limit': 50000,
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp'
                }
            }
        resp = self.client.slb.virtual_server.vport.update(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=0,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            virtual_port_templates={
                'template-virtual-port': 'template_vp'
            },
            use_rcv_hop=False,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_tcp_virtual_port_update_with_templates(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.POST, OBJECT_TCP_URL, json=json_response, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'ipinip': 1,
                'no-dest-nat': 0,
                'use-rcv-hop-for-resp': 0,
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'tcp',
                'service-group': 'pool1',
                'ha-conn-mirror': 1,
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'template-virtual-port': 'template_vp'
            }
        }
        resp = self.client.slb.virtual_server.vport.update(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.TCP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=0,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            virtual_port_templates={
                'template-virtual-port': 'template_vp'
            },
            use_rcv_hop=False,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, OBJECT_TCP_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_replace_with_params(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.PUT, OBJECT_URL, json=json_response, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'ipinip': 1,
                'no-dest-nat': 1,
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'http',
                'service-group': 'pool1',
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'use-rcv-hop-for-resp': 1,
            }
        }

        resp = self.client.slb.virtual_server.vport.replace(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            use_rcv_hop=True,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.PUT)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_tcp_virtual_port_replace_with_params(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.PUT, OBJECT_TCP_URL, json=json_response, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'ipinip': 1,
                'no-dest-nat': 1,
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'tcp',
                'service-group': 'pool1',
                'ha-conn-mirror': 1,
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'use-rcv-hop-for-resp': 1,
            }
        }

        resp = self.client.slb.virtual_server.vport.replace(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.TCP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=1,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            use_rcv_hop=True,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.PUT)
        self.assertEqual(responses.calls[1].request.url, OBJECT_TCP_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_virtual_port_replace_with_templates(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.PUT, OBJECT_URL, json=json_response, status=200)
        protocol = self.client.slb.virtual_server.vport.HTTP
        if protocol.lower() == 'http':
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'name': 'test1_VPORT',
                    'ipinip': 1,
                    'no-dest-nat': 0,
                    'use-rcv-hop-for-resp': 0,
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'conn-limit': 50000,
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp'
                }
            }
        else:
            params = {
                'port':
                {
                    'auto': 1,
                    'extended-stats': 1,
                    'name': 'test1_VPORT',
                    'ipinip': 1,
                    'no-dest-nat': 0,
                    'use-rcv-hop-for-resp': 0,
                    'pool': 'test_nat_pool',
                    'port-number': 80,
                    'protocol': 'http',
                    'service-group': 'pool1',
                    'ha-conn-mirror': 1,
                    'conn-limit': 50000,
                    'tcp_template': 'test_tcp_template',
                    'template-persist-cookie': 'test_c_pers_template',
                    'template-persist-source-ip': 'test_s_pers_template',
                    'udp_template': 'test_udp_template',
                    'template-virtual-port': 'template_vp'
                }
            }
        resp = self.client.slb.virtual_server.vport.replace(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.HTTP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=0,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            virtual_port_templates={
                'template-virtual-port': 'template_vp'
            },
            use_rcv_hop=False,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.PUT)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @mock.patch('acos_client.v30.slb.virtual_port.VirtualPort.get')
    @responses.activate
    def test_tcp_virtual_port_replace_with_templates(self, mocked_get):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        mocked_get.return_value = {"foo": "bar"}
        json_response = {"foo": "bar"}
        responses.add(responses.PUT, OBJECT_TCP_URL, json=json_response, status=200)
        params = {
            'port':
            {
                'auto': 1,
                'extended-stats': 1,
                'name': 'test1_VPORT',
                'ipinip': 1,
                'no-dest-nat': 0,
                'use-rcv-hop-for-resp': 0,
                'pool': 'test_nat_pool',
                'port-number': 80,
                'protocol': 'tcp',
                'service-group': 'pool1',
                'ha-conn-mirror': 1,
                'conn-limit': 50000,
                'tcp_template': 'test_tcp_template',
                'template-persist-cookie': 'test_c_pers_template',
                'template-persist-source-ip': 'test_s_pers_template',
                'udp_template': 'test_udp_template',
                'template-virtual-port': 'template_vp'
            }
        }
        resp = self.client.slb.virtual_server.vport.replace(
            virtual_server_name=VSERVER_NAME,
            name='test1_VPORT',
            protocol=self.client.slb.virtual_server.vport.TCP,
            protocol_port='80',
            service_group_name='pool1',
            s_pers_name="test_s_pers_template",
            c_pers_name="test_c_pers_template",
            status=1,
            autosnat=True,
            ipinip=True,
            ha_conn_mirror=1,
            no_dest_nat=0,
            conn_limit=50000,
            source_nat_pool="test_nat_pool",
            tcp_template="test_tcp_template",
            udp_template="test_udp_template",
            virtual_port_templates={
                'template-virtual-port': 'template_vp'
            },
            use_rcv_hop=False,
        )
        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.PUT)
        self.assertEqual(responses.calls[1].request.url, OBJECT_TCP_URL)
        self.assertEqual(json.loads(responses.calls[1].request.body), params)

    @responses.activate
    def test_virtual_port_delete(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.DELETE, OBJECT_URL, json=OK_RESP, status=200)

        resp = self.client.slb.virtual_server.vport.delete(
            VSERVER_NAME, 'test1_VPORT', self.client.slb.virtual_server.vport.HTTP, '80'
        )

        self.assertEqual(resp, OK_RESP)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.DELETE)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_virtual_port_delete_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 1043, "msg": "Can not find the virtual server port"}}
        }
        responses.add(responses.DELETE, OBJECT_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.ACOSException):
            self.client.slb.virtual_server.vport.delete(
                VSERVER_NAME, 'test1_VPORT', self.client.slb.virtual_server.vport.HTTP, '80'
            )

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.DELETE)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_virtual_port_search(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, OBJECT_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.vport.get(
            VSERVER_NAME, 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, port='80'
        )

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_virtual_port_search_not_found(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {
            "response": {"status": "fail", "err": {"code": 1043, "msg": "Can not find the virtual server port"}}
        }
        responses.add(responses.GET, OBJECT_URL, json=json_response, status=200)

        with self.assertRaises(acos_errors.ACOSException):
            self.client.slb.virtual_server.vport.get(
                VSERVER_NAME, 'test1_VPORT', protocol=self.client.slb.virtual_server.vport.HTTP, port='80'
            )

        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, OBJECT_URL)

    @responses.activate
    def test_virtual_port_all(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        json_response = {"foo": "bar"}
        responses.add(responses.GET, ALL_URL, json=json_response, status=200)

        resp = self.client.slb.virtual_server.vport.all(VSERVER_NAME)

        self.assertEqual(resp, json_response)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.GET)
        self.assertEqual(responses.calls[1].request.url, ALL_URL)



# Copyright 2016, All Rights Reserved,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import six

CLEAN_FIELDS = ["username", "password"]

REPLACEMENT = "*" * 8


def clean(data, field=None):
    if field in CLEAN_FIELDS:
        return REPLACEMENT

    # Mocks are gross and they don't live in production code.
    # We can ignore them.
    if type(data).__module__ == 'mock.mock':
        return data

    if type(data) is dict:
        return type(data)(
            (x, clean(y, field=x)) for x, y in six.iteritems(data)
        )
    elif isinstance(data, six.string_types):
        return data
    elif isinstance(data, (list, tuple)):
        return type(data)(clean(x) for x in data)

    return data



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base


class Admin(base.BaseV21):

    @property
    def administrator(self):
        return self.Administrator(self.client)

    class Administrator(base.BaseV21):

        def all(self, **kwargs):
            return self._get('system.admin.administrator.getAll', **kwargs)

        def get(self, name, **kwargs):
            params = {"admin_name": name}
            return self._post('system.admin.administrator.search', params, **kwargs)

        def create(self, name, **kwargs):
            params = {
                "administrator": {
                    "admin_name": name
                }
            }

            return self._post('system.admin.administrator.create', params, **kwargs)

        def update(self, name, **kwargs):
            params = {
                "administrator": {
                    "admin_name": name
                }
            }

            return self._post('system.admin.administrator.update', params, **kwargs)

        def delete(self, name, **kwargs):
            params = {"admin_name": name}
            return self._post('system.admin.administrator.delete', params, **kwargs)

        def all_delete(self, **kwargs):
            return self._post('system.admin.administrator.deleteAll', **kwargs)



# Copyright 2015,  Tobit Raff,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals
import six


from acos_client import errors as acos_errors
from acos_client.v30 import base


class BaseSSL(base.BaseV30):

    def get(self, name, **kwargs):
        return self._get(self.url_prefix + name, **kwargs)

    def exists(self, name):
        try:
            self.get(name)
            return True
        except acos_errors.NotFound:
            return False

    def _set(self, name, cert="", key="", passphrase="", update=False,
             cipher_template=None, **kwargs):
        # Unimplemented options:
        # encrypted, session_ticket_enable, version, forward_proxy_enable,
        # close_notify, session_cache_size, session_cache_timeout,
        # server_certificate_error, cipher_without_prio_list,

        params = {}

        if cipher_template:
            params = {
                "cipher": {
                    "name": name,
                    "cipher-cfg": cipher_template
                }
            }

        else:
            obj_params = {
                "name": name,
                "cert": cert,
                "key": key,
                self.passphrase: passphrase,
                # Unimplemented options:
                # "encrypted": encrypted,
                # "session-ticket-enable": session_ticket_enable,
                # "version": version,
                # "forward-proxy-enable": forward_proxy_enable,
                # "close-notify": close_notify,
                # "session-cache-size": session_cache_size,
                # "session-cache-timeout": session_cache_timeout,
                # "server-certificate-error": server_certificate_error,
                # "cipher-without-prio-list": cipher_without_prio_list,
                # "ca-certs": ca_certs,
            }

            params = {'%s-ssl' % self.prefix: {}}
            for key, val in six.iteritems(obj_params):
                # Filter out invalid, or unset keys
                if val != "":
                    params['%s-ssl' % self.prefix][key] = val

            if not update:
                name = ''

        self._post(self.url_prefix + name, params, **kwargs)

    def create(self, name, cert="", key="", passphrase="", cipher_template=None, **kwargs):
        if self.exists(name):
            raise acos_errors.Exists

        self._set(name, cert, key, passphrase, cipher_template=cipher_template, **kwargs)

    def update(self, name, cert="", key="", passphrase="", **kwargs):
        self._set(name, cert, key, passphrase, update=True, **kwargs)

    def delete(self, name, **kwargs):
        self._delete(self.url_prefix + name, **kwargs)


class ClientSSL(BaseSSL):

    url_prefix = '/slb/template/client-ssl/'
    prefix = 'client'
    passphrase = 'key-passphrase'


class ServerSSL(BaseSSL):

    url_prefix = '/slb/template/server-ssl/'
    prefix = 'server'
    passphrase = 'passphrase'


class SSLCipher(BaseSSL):

    url_prefix = '/slb/template/cipher/'
    passphrase = None
    prefix = None



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

import json
import logging
from requests.adapters import HTTPAdapter
from requests import Session
import six
import sys

import acos_client
from acos_client import logutils
from acos_client.v21 import responses as acos_responses
from acos_client.v21.ssl_adapter import SSLAdapter

LOG = logging.getLogger(__name__)

out_hdlr = logging.StreamHandler(sys.stderr)
out_hdlr.setLevel(logging.ERROR)
LOG.addHandler(out_hdlr)
LOG.setLevel(logging.ERROR)


def extract_method(api_url):
    q = six.moves.urllib_parse.urlparse(api_url).query
    return six.moves.urllib_parse.parse_qs(q).get('method', [''])[0]


def merge_dicts(d1, d2):
    d = d1.copy()
    # if isinstance(d1, dict) else {}
    for k, v in d2.items():
        if k in d and isinstance(d[k], dict):
            d[k] = merge_dicts(d[k], d2[k])
        else:
            d[k] = d2[k]
    return d


broken_replies = {
    ('<?xml version="1.0" encoding="utf-8" ?><response status="ok">'
     '</response>'): (json.dumps({"response": {"status": "OK"}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" Partition does not exist. '
     '(internal error: 520749062)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 999,
     "msg": " Partition does not exist."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" Failed to get partition. (internal error: '
     '402718800)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 999,
     "msg": " Partition does not exist."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="1076" msg="Invalid partition parameter." /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 1076,
     "msg": "Invalid partition parameter."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" No such aFleX. (internal error: '
     '17039361)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 17039361,
     "msg": " No such aFleX."}}})),

    ('<?xml version="1.0" encoding="utf-8" ?><response status="fail">'
     '<error code="999" msg=" This aFleX is in use. (internal error: '
     '17039364)" /></response>'):
    (json.dumps({"response": {"status": "fail", "err": {"code": 17039364,
     "msg": " This aFleX is in use."}}})),

}


class HttpClient(object):
    HEADERS = {
        "Content-type": "application/json",
        "User-Agent": "ACOS-Client-AGENT-%s" % acos_client.VERSION,
    }

    def __init__(self, host, port=None, protocol="https", max_retries=3, timeout=5):
        if port is None:
            if protocol == 'http':
                self.port = 80
            else:
                self.port = 443
        else:
            self.port = port
        self.url_base = "%s://%s:%s" % (protocol, host, self.port)
        self.max_retries = max_retries
        self.timeout = timeout

    def request(self, method, api_url, params={}, **kwargs):
        """Generate the API call to the device."""

        LOG.debug("axapi_http: full url = %s", self.url_base + api_url)
        LOG.debug("axapi_http: %s url = %s", method, api_url)
        LOG.debug("axapi_http: params = %s", json.dumps(logutils.clean(params), indent=4))

        # Set "data" variable for the request
        if params:
            extra_params = kwargs.get('axapi_args', {})
            params_copy = merge_dicts(params, extra_params)
            LOG.debug("axapi_http: params_all = %s", logutils.clean(params_copy))

            payload = json.dumps(params_copy)
        else:
            try:
                payload = kwargs.pop('payload', None)
                self.headers = dict(self.HEADERS, **kwargs.pop('headers', {}))
                LOG.debug("axapi_http: headers_all = %s", logutils.clean(self.headers))
            except KeyError:
                payload = None

        max_retries = kwargs.get('max_retries', self.max_retries)
        timeout = kwargs.get('timeout', self.timeout)

        # Create session to set HTTPAdapter or SSLAdapter
        session = Session()
        if self.port == 443:
            # Add adapter for any https session to force TLS1_0 connection for v21 of AXAPI
            session.mount('https://', SSLAdapter(max_retries=max_retries))
        else:
            session.mount('http://', HTTPAdapter(max_retries=max_retries))
        session_request = getattr(session, method.lower())

        # Make actual request and handle any errors
        try:
            device_response = session_request(
                self.url_base + api_url, verify=False, data=payload, headers=self.HEADERS, timeout=timeout
            )
        except (Exception) as e:
            LOG.error("acos_client failing with error %s after %s retries", e.__class__.__name__, max_retries)
            raise e
        finally:
            session.close()

        # Log if the reponse is one of the known broken response
        if device_response in broken_replies:
            device_response = broken_replies[device_response]
            LOG.debug("axapi_http: broken reply, new response: %s", logutils.clean(device_response))

        # Validate json response
        try:
            json_response = device_response.json()
            LOG.debug("axapi_http: data = %s", json.dumps(logutils.clean(json_response), indent=4))
        except ValueError as e:
            # The response is not JSON but it still succeeded.
            LOG.debug("axapi_http: json = %s", e)
            return device_response

        # Handle "fail" responses returned by AXAPI
        if 'response' in json_response and 'status' in json_response['response']:
            if json_response['response']['status'] == 'fail':
                    acos_responses.raise_axapi_ex(json_response, action=extract_method(api_url))

        # Return json portion of response
        return json_response

    def get(self, api_url, params={}, **kwargs):
        return self.request("GET", api_url, params, **kwargs)

    def post(self, api_url, params={}, **kwargs):
        return self.request("POST", api_url, params, **kwargs)



# Copyright (C) 2021, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from acos_client import errors as acos_errors
from acos_client.v30 import base


class LicenseRequest(base.BaseV30):
    url_prefix = '/glm/create-license-request'

    def _set(self, create_license_request):
        params = {
            'create-license-request': create_license_request
        }

        return params

    def create(self, create_license_request=None, **kwargs):
        params = self._set(create_license_request=None)
        return self._post(self.url_prefix, params, axapi_args=kwargs)

    def update(self, create_license_request=None, **kwargs):
        params = self._set(create_license_request=None)
        return self._post(self.url_prefix, params, axapi_args=kwargs)

    def put(self, create_license_request=None, **kwargs):
        params = self._set(create_license_request=None)
        return self._put(self.url_prefix, params, axapi_args=kwargs)

    def delete(self):
        return self._delete(self.url_prefix)


class MultiLicenseException(Exception):

    def __init__(self):
        self.message = ("Only one of the following attributes can be "
                        "used to define a new license: existing_org, "
                        "existing_user, new_user, or name. These cannot "
                        "be used in conjuction.")
        super(MultiLicenseException, self).__init__()


class NewLicense(base.BaseV30):
    url_prefix = '/glm/new-license'

    def create(self, account_name=None, country=None, existing_org=None,
               glm_password=None, last_name=None, name=None, new_email=None,
               new_password=None, new_user=None, org_id=None, phone=None,
               license_type=None, existing_user=None, first_name=None,
               glm_email=None):
        params = {
            "new-license": {}
        }

        xor = bool(existing_org) + bool(existing_user) + bool(new_user) + bool(name)

        if xor > 1:
            raise MultiLicenseException()

        if existing_org:
            params['new-license'] = self.minimal_dict({
                'existing-org': existing_org,
                'org-id': org_id
            })
        elif existing_user:
            if not glm_email:
                raise acos_errors.RequiredAttributeNotSpecified(
                    self.url_prefix, "existing_user", ["glm_email"])

            params['new-license'] = self.minimal_dict({
                'existing-user': existing_user,
                'glm-email': glm_email,
                'glm-password': glm_password
            })
        elif new_user:
            if not new_email:
                raise acos_errors.RequiredAttributeNotSpecified(
                    self.url_prefix, "new_user", ["new_email"])

            params['new-license'] = self.minimal_dict({
                'new-user': new_user,
                'new-email': new_email,
                'new-password': new_password,
                'account-name': account_name,
                'first-name': first_name,
                'last-name': last_name,
                'country': country,
                'phone': phone
            })
        elif name:
            if not license_type:
                raise acos_errors.RequiredAttributeNotSpecified(
                    self.url_prefix, "name", ["license_type"])

            params['new-license'] = self.minimal_dict({
                'name': name,
                'type': license_type
            })

        self._post(self.url_prefix, params)



# Copyright (C) 2021, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from acos_client.v30 import base


class SecretStringUndefinedException(Exception):

    def __init__(self):
        self.message = ("The secret_string argument must "
                        "be defined if password is specified.")
        super(SecretStringUndefinedException, self).__init__(self.message)


class ProxyServer(base.BaseV30):
    url_prefix = '/glm/proxy-server'

    def _set(self, host=None, port=None, username=None,
             password=None, secret_string=None):
        params = {
            'proxy-server': {
                'host': host,
                'port': port,
                'username': username,
                'password': password,
                'secret-string': secret_string
            }
        }

        if password and not secret_string:
            raise SecretStringUndefinedException()

        return params

    def create(self, host=None, port=None, username=None,
               password=None, secret_string=None, **kwargs):
        params = self._set(host=host, port=port, username=username,
                           password=password, secret_string=secret_string)
        return self._post(self.url_prefix, params, axapi_args=kwargs)

    def update(self, host=None, port=None, username=None,
               password=None, secret_string=None, **kwargs):
        params = self._set(host=host, port=port, username=username,
                           password=password, secret_string=secret_string)
        return self._post(self.url_prefix, params, axapi_args=kwargs)

    def put(self, host=None, port=None, username=None,
            password=None, secret_string=None, **kwargs):
        params = self._set(host=host, port=port, username=username,
                           password=password, secret_string=secret_string)
        return self._put(self.url_prefix, params, axapi_args=kwargs)

    def delete(self):
        return self._delete(self.url_prefix)



# Copyright 2015, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base


class DeviceContext(base.BaseV30):
    """Switching of device-contexts"""
    url_prefix = "/device-context"

    def switch(self, device_id, obj_slot_id):
        """Switching of device-context"""
        payload = {
            "device-context": self._build_payload(device_id, obj_slot_id)
        }

        return self._post(self.url_prefix, payload)

    def _build_payload(self, device_id, obj_slot_id):

        payload = {
            "device-id": device_id
        }

        return payload



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base


class LicenseManager(base.BaseV21):
    """v2.1 LicenseManager is not yet supported"""
    def create(self, host_list=[], serial=None, instance_name=None, use_mgmt_port=False,
               interval=None, bandwidth_base=None, bandwidth_unrestricted=None):
        raise NotImplementedError("LicenseManager is not yet supported using AXAPI v2.1")

    def get(self):
        raise NotImplementedError("LicenseManager is not yet supported using AXAPI v2.1")

    def connect(self, connect=False):
        raise NotImplementedError("LicenseManager is not yet supported using AXAPI v2.1")

    def update(self, host_list=[], serial=None, instance_name=None, use_mgmt_port=False,
               interval=None, bandwidth_base=None, bandwidth_unrestricted=None):
        raise NotImplementedError("LicenseManager is not yet supported using AXAPI v2.1")



# Copyright (C) 2021, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v30 import base


class DeleteGLMLicense(base.BaseV30):
    url_prefix = '/delete/glm-license/'

    def post(self, a10_ti=None, ipsec_vpn=None, qosmos=None,
             threatstop=None, webroot=None, webroot_ti=None,
             max_retries=None, timeout=None, **kwargs):
        params = {
            "glm-license": self.minimal_dict({
                "a10-ti": a10_ti,
                "ipsec-vpn": ipsec_vpn,
                "qosmos": qosmos,
                "threatstop": threatstop,
                "webroot": webroot,
                "webroot-ti": webroot_ti
            }),
        }

        if not params['glm-license']:
            params = {}

        self._post(self.url_prefix, params, max_retries=max_retries,
                   timeout=timeout, axapi_args=kwargs)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import absolute_import
from __future__ import unicode_literals

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from acos_client import client
import responses


HOSTNAME = 'fake_a10'
BASE_URL = "https://{}:443/services/rest/v2.1/?format=json&method=".format(HOSTNAME)
AUTH_URL = "{}authenticate".format(BASE_URL)
HA_URL = '{}ha.sync_config&session_id={}'.format(BASE_URL, 'foobar')


class TestHighAvailability(unittest.TestCase):

    def setUp(self):
        self.client = client.Client(HOSTNAME, '21', 'fake_username', 'fake_password')

    @responses.activate
    def test_high_availability_sync(self):
        responses.add(responses.POST, AUTH_URL, json={'session_id': 'foobar'})
        responses.add(responses.POST, HA_URL)

        resp = self.client.ha.sync('192.168.2.254', 'fake_username', 'fake_password')

        self.assertIsNone(resp)
        self.assertEqual(len(responses.calls), 2)
        self.assertEqual(responses.calls[1].request.method, responses.POST)
        self.assertEqual(responses.calls[1].request.url, HA_URL)



# flake8: noqa

# Classes for using multipart form data from Python, which does not (at the
# time of writing) support this directly.

# To use this, make an instance of Multipart and add parts to it via the
# factory methods field and file.  When you are done, get the content via
# the get method.

# @author: Stacy Prowell (http://stacyprowell.com)

# Public domain:

# On 1/14/15, 9:58 AM, "Prowell, Stacy J." <prowellsj@ornl.gov> wrote:

# Doug,

# I am excited that someone found it useful.  At present it has no license
# attached; I regard it as public domain, so you are welcome to use it as
# you see fit.  If it is *better* that it have a license (so you can clearly
# indicate the use of the external code), then I can attach a compatible
# license.  Just let me know!

# Kind regards,
# Stacy
from __future__ import absolute_import, unicode_literals

import mimetypes


class Part(object):
    # '''
    # Class holding a single part of the form.  You should never need to use
    # this class directly; instead, use the factory methods in Multipart:
    # field and file.
    # '''

    # The boundary to use.  This is shamelessly taken from the standard.
    BOUNDARY = '----------AaB03x'
    CRLF = '\r\n'
    # Common headers.
    CONTENT_TYPE = 'Content-Type'
    CONTENT_DISPOSITION = 'Content-Disposition'
    # The default content type for parts.
    DEFAULT_CONTENT_TYPE = 'application/octet-stream'

    def __init__(self, name, filename, body, headers):
        # '''
        # Make a new part.  The part will have the given headers added
        # initially.

        # @param name: The part name.
        # @type name: str
        # @param filename: If this is a file, the name of the file.  Otherwise
        #                 None.
        # @type filename: str
        # @param body: The body of the part.
        # @type body: str
        # @param headers: Additional headers, or overrides, for this part.
        #                 You can override Content-Type here.
        # @type headers: dict
        # '''
        self._headers = headers.copy()
        self._name = name
        self._filename = filename
        self._body = body
        # We respect any content type passed in, but otherwise set it here.
        # We set the content disposition now, overwriting any prior value.
        if self._filename is None:
            self._headers[Part.CONTENT_DISPOSITION] = \
                ('form-data; name="%s"' % self._name)
            self._headers.setdefault(Part.CONTENT_TYPE,
                                     Part.DEFAULT_CONTENT_TYPE)
        else:
            self._headers[Part.CONTENT_DISPOSITION] = \
                ('form-data; name="%s"; filename="%s"' %
                 (self._name, self._filename))
            self._headers.setdefault(Part.CONTENT_TYPE,
                                     mimetypes.guess_type(filename)[0]
                                     or Part.DEFAULT_CONTENT_TYPE)
        return

    def get(self):
        # '''
        # Convert the part into a list of lines for output.  This includes
        # the boundary lines, part header lines, and the part itself.  A
        # blank line is included between the header and the body.

        # @return: Lines of this part.
        # @rtype: list
        # '''
        lines = []
        lines.append('--' + Part.BOUNDARY)
        for (key, val) in self._headers.items():
            lines.append(str('%s: %s' % (key, val)))
        lines.append('')
        lines.append(self._body)
        return lines


class Multipart(object):
    # '''
    # Encapsulate multipart form data.  To use this, make an instance and then
    # add parts to it via the two methods (field and file).  When done, you can
    # get the result via the get method.

    # See http://www.w3.org/TR/html401/interact/forms.html#h-17.13.4.2 for
    # details on multipart/form-data.

    # Watch http://bugs.python.org/issue3244 to see if this is fixed in the
    # Python libraries.

    # @return: content type, body
    # @rtype: tuple
    # '''

    def __init__(self):
        self.parts = []
        return

    def field(self, name, value, headers={}):
        # '''
        # Create and append a field part.  This kind of part has a field name
        # and value.

        # @param name: The field name.
        # @type name: str
        # @param value: The field value.
        # @type value: str
        # @param headers: Headers to set in addition to disposition.
        # @type headers: dict
        # '''
        self.parts.append(Part(name, None, value, headers))
        return

    def file(self, name, filename, value, headers={}):
        # '''
        # Create and append a file part.  THis kind of part has a field name,
        # a filename, and a value.

        # @param name: The field name.
        # @type name: str
        # @param value: The field value.
        # @type value: str
        # @param headers: Headers to set in addition to disposition.
        # @type headers: dict
        # '''
        self.parts.append(Part(name, filename, value, headers))
        return

    def get(self):
        # '''
        # Get the multipart form data.  This returns the content type, which
        # specifies the boundary marker, and also returns the body containing
        # all parts and bondary markers.

        # @return: content type, body
        # @rtype: tuple
        # '''
        all = []
        for part in self.parts:
            all += part.get()
        all.append('--' + Part.BOUNDARY + '--')
        all.append('')
        # We have to return the content type, since it specifies the boundary.
        content_type = 'multipart/form-data; boundary=%s' % Part.BOUNDARY
        return content_type, Part.CRLF.join(all)



# Copyright (C) 2021, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from acos_client.v30 import base
from acos_client.v30.glm import license
from acos_client.v30.glm import proxy


class Flexpool(base.BaseV30):
    url_prefix = '/glm'

    @property
    def proxy_server(self):
        return proxy.ProxyServer(self.client)

    @property
    def create_license_request(self):
        return license.LicenseRequest(self.client)

    @property
    def new_licese(self):
        return license.NewLicense(self.client)

    @property
    def send(self):
        return Send(self.client)

    def _set(self, appliance_name=None, allocate_bandwidth=None, burst=None,
             check_expiration=None, enable_requests=None, enterpise=None,
             enterprise_request_type=None, host=None, interval=None,
             port=None, thunder_capacity_license=None, token=None,
             use_mgmt_port=None):

        params = {
            "glm": self.minimal_dict({
                "host": host,
                "port": port,
                "burst": burst,
                "token": token,
                "interval": interval,
                "enterpise": enterpise,
                "use-mgmt-port": use_mgmt_port,
                "appliance-name": appliance_name,
                "enable-requests": enable_requests,
                "check-expiration": check_expiration,
                "allocate-bandwidth": allocate_bandwidth,
                "enterprise-request-type": enterprise_request_type,
                "thunder-capacity-license": thunder_capacity_license,
            })
        }
        return params

    def create(self, appliance_name=None, allocate_bandwidth=None, burst=None,
               check_expiration=None, enable_requests=None, enterpise=None,
               enterprise_request_type=None, host=None, interval=None,
               port=None, thunder_capacity_license=None, token=None,
               use_mgmt_port=None, **kwargs):
        params = self._set(appliance_name=appliance_name,
                           allocate_bandwidth=allocate_bandwidth, burst=burst,
                           check_expiration=check_expiration, enable_requests=enable_requests,
                           enterpise=enterpise, enterprise_request_type=enterprise_request_type,
                           thunder_capacity_license=thunder_capacity_license,
                           token=token, use_mgmt_port=use_mgmt_port,
                           host=host, interval=interval, port=port)
        return self._post(self.url_prefix, params, axapi_args=kwargs)

    def update(self, appliance_name=None, allocate_bandwidth=None, burst=None,
               check_expiration=None, enable_requests=None, enterpise=None,
               enterprise_request_type=None, host=None, interval=None,
               port=None, thunder_capacity_license=None, token=None,
               use_mgmt_port=None, **kwargs):
        params = self._set(appliance_name=appliance_name,
                           allocate_bandwidth=allocate_bandwidth, burst=burst,
                           check_expiration=check_expiration, enable_requests=enable_requests,
                           enterpise=enterpise, enterprise_request_type=enterprise_request_type,
                           thunder_capacity_license=thunder_capacity_license,
                           token=token, use_mgmt_port=use_mgmt_port,
                           host=host, interval=interval, port=port)
        return self._post(self.url_prefix, params, axapi_args=kwargs)

    def replace(self, appliance_name=None, allocate_bandwidth=None, burst=None,
                check_expiration=None, enable_requests=None, enterpise=None,
                enterprise_request_type=None, host=None, interval=None,
                port=None, thunder_capacity_license=None, token=None,
                use_mgmt_port=None, **kwargs):
        params = self._set(appliance_name=appliance_name,
                           allocate_bandwidth=allocate_bandwidth, burst=burst,
                           check_expiration=check_expiration, enable_requests=enable_requests,
                           enterpise=enterpise,
                           enterprise_request_type=enterprise_request_type,
                           thunder_capacity_license=thunder_capacity_license,
                           token=token, use_mgmt_port=use_mgmt_port,
                           host=host, interval=interval, port=port)
        return self._put(self.url_prefix, params, axapi_args=kwargs)

    def delete(self):
        return self._delete(self.url_prefix)


class Send(base.BaseV30):
    url_prefix = '/glm/send'

    def create(self, license_request):
        params = {
            "send": self.minimal_dict({
                "license-request": license_request
            })
        }

        self._post(self.url_prefix, params)



# Copyright 2016, A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from acos_client import errors as acos_errors
from acos_client.v30 import base
from acos_client.v30.vrrpa.blade_params import BladeParameters


class VRID(base.BaseV30):

    def __init__(self, client):
        super(VRID, self).__init__(client)
        self.client = client
        self.base_url = "/vrrp-a/vrid/"

    @property
    def blade(self):
        return BladeParameters(self.client)

    def get(self, vrid_val):
        return self._get(self.base_url + str(vrid_val))

    def exists(self, vrid_val):
        try:
            self.get(vrid_val)
            return True
        except acos_errors.NotFound:
            return False

    def _build_params(self, vrid_val, threshold=None, disable=None, floating_ips=[],
                      is_partition=False):
        vrid = {'vrid-val': vrid_val}
        vrid_floating_ips = None
        if floating_ips:
            if is_partition:
                ip_partition_list = [{'ip-address-partition': ip} for ip in floating_ips]
                vrid_floating_ips = {
                    'ip-address-part-cfg': ip_partition_list
                }
            else:
                ip_list = [{'ip-address': ip} for ip in floating_ips]
                vrid_floating_ips = {
                    'ip-address-cfg': ip_list
                }
            vrid['floating-ip'] = vrid_floating_ips

        if threshold or disable:
            threshold = threshold if threshold in range(0, 256) else 1
            disable = disable if disable in [0, 1] else 0
            preempt = {
                'threshold': threshold,
                'disable': disable
            }
            vrid['preempt-mode'] = preempt

        payload = {'vrid': vrid}
        return payload

    def create(self, vrid_val, threshold=None, disable=None, floating_ips=[], is_partition=False):
        return self._post(self.base_url, self._build_params(vrid_val, threshold, disable,
                                                            floating_ips, is_partition))

    def update(self, vrid_val, threshold=None, disable=None, floating_ips=[], is_partition=False):
        return self._put(self.base_url + str(vrid_val), self._build_params(vrid_val, threshold,
                                                                           disable, floating_ips,
                                                                           is_partition))

    def delete(self, vrid_val):
        return self._delete(self.base_url + str(vrid_val))



# Copyright (C) 2016, A10 Networks Inc. All rights reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import
from __future__ import unicode_literals

from acos_client.v21 import base


class DNS(base.BaseV21):

    def set(self, primary=None, secondary=None, suffix=None, **kwargs):
        settings = {}

        if primary is not None:
            settings['primary_dns'] = primary

        if secondary is not None:
            settings['secondary_dns'] = secondary

        if suffix is not None:
            settings['dns_suffix'] = suffix

        payload = {'dns': settings}

        return self._post("network.dns.server.set", payload, **kwargs)
